function [U,S] = fast_pca(matrix,h)
## /**********************************************************************
##   Copyright 2013, Hanchen Xiong
##   email: hanchen.xiong@uibk.ac.at
##
##   This file is part of 3D Point Registration Benchmark Framework
##
##   This is free software: you can redistribute it and/or modify
##   it under the terms of the GNU Lesser General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This code is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU Lesser General Public License for more details.
##
##   You can find a copy of the GNU Lesser General Public License
##   in <http://www.gnu.org/licenses/>.
##
## ***********************************************************************/

%--------------------------------------------------------------------------
% fast pca implementation based on 
% Sharma, Alokanand and Paliwal, K.K. (2007) Fast principal component 
% analysis using fixed-point algorithm. Pattern Recognition Letters, 28 
% (10). pp. 1151-1155. ISSN 0167-8655
%--------------------------------------------------------------------------
d=size(matrix,1);
U=zeros(d,h);     % top h eigenvectors with largest eigenvalues
S=zeros(h);       % top h eigenvalues
for i=1:h
    u=rand(d,1);
    while 1
        u_old=u;
        u=matrix*u;
        temp=zeros(d,1);
        for j=1:i-1
            temp=temp+(u'*U(:,j))*U(:,j);
        end;
        u=u-temp;
        lambda=norm(u);
        u=u/lambda;

        if abs(u'*u_old-1)<0.000001
            U(:,i)=u;
            S(i)=lambda;
            matrix=matrix-lambda*u*u';
            break;
        end;
    end;
end;
end

