function[]=motion_experiment(data_directory,vis,save_file)
## /**********************************************************************
##   Copyright 2013, Hanchen Xiong
##   email: hanchen.xiong@uibk.ac.at
##
##   This file is part of 3D Point Registration Benchmark Framework
##
##   This is free software: you can redistribute it and/or modify
##   it under the terms of the GNU Lesser General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This code is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU Lesser General Public License for more details.
##
##   You can find a copy of the GNU Lesser General Public License
##   in <http://www.gnu.org/licenses/>.
##
## ***********************************************************************/


% data_directory: the directory in which all points clouds are organised with different sizes;
% vis: 1- visualize the alignment; 0- no visualization
% save_file: 1- save all visualized alignments
%--------------------------------------------------------------------------
current_path=pwd;                                       % current full path
data_directory=[current_path,data_directory];           % full directory
%----------check if the input is correct form of a directory---------------
if ~isdir(data_directory)
    disp('the input is not a directory');
    return; 
end;

mkdir('motion_experiment');

%------------------extract information from input-------------------------- 
cd (data_directory);               % go to the specified directory
filename=strcat('*.','txt');          % find all '.txt' files
filename=dir(filename);               % extract the corresponding file names
num_D=size(filename,1);               % the number of point_cloud files in the category

error_ICP=zeros(num_D,6);
error_GM=zeros(num_D,6);
error_SoftAssign=zeros(num_D,6);
error_PCA=zeros(num_D,6);

cd (current_path);
for j=1:num_D
    points=load([data_directory,'/',filename(j,1).name]);
    translation_factor=std(points);
    rotat_factor=[pi/6,pi/36,pi/36];
    for i=1:6
        [source,target]=datagenerator([data_directory,'/',filename(j,1).name], i*rotat_factor(1), i*rotat_factor(2),i*rotat_factor(3),i*translation_factor(1),i*translation_factor(2),i*translation_factor(3), 0);
       
           
       %---------------------------visualization of the alignment-------------------------------
       if vis==1
           figure;
               set(gcf,'position',[400,200,1550,500]);               
               subplot(1,5,1);
               scatter3(target(:,1),target(:,2),target(:,3),10,'filled','black'); hold on;
               scatter3(source(:,1),source(:,2),source(:,3),10,'red');
               set(gca,'xticklabel',[]);
               set(gca,'yticklabel',[]);
               set(gca,'zticklabel',[]);
               axis equal; axis vis3d;
               grid off;
               view(60,40);
               title('Point clouds before alignment');
               
               [TR,TT]=icp(target',source');
               pose=[TR,TT; 0 0 0 1];
               size_source=size(source,1);
               transformed=pose*[source,ones(size_source,1)]'; 
               transformed=transformed(1:3,:)';
               subplot(1,5,2); 
               scatter3(target(:,1),target(:,2),target(:,3),10,'filled','black'); hold on;
               scatter3(transformed(:,1),transformed(:,2),transformed(:,3),9,'red');
               set(gca,'xticklabel',[]);
               set(gca,'yticklabel',[]);
               set(gca,'zticklabel',[]);
               axis equal; axis vis3d;
               grid off;
               view(60,40);
               err=error_cal(target,transformed);
               title(['ICP error is:',num2str(err)]);
               error_ICP(j,i)=err;

               [transformed]=GaussianMixture(source,target,0);
               subplot(1,5,3); 
               scatter3(target(:,1),target(:,2),target(:,3),10,'filled','black'); hold on;
               scatter3(transformed(:,1),transformed(:,2),transformed(:,3),9,'red');
               set(gca,'xticklabel',[]);
               set(gca,'yticklabel',[]);
               set(gca,'zticklabel',[]);
               axis equal; axis vis3d;
               grid off;
               view(60,40);
               err=error_cal(target,transformed);
               title(['GaussianMixture error is:',num2str(err)]);
               error_GM(j,i)=err;

               [transformed]=SoftAssign(source,target,0);
               subplot(1,5,4); 
               scatter3(target(:,1),target(:,2),target(:,3),10,'filled','black'); hold on;
               scatter3(transformed(:,1),transformed(:,2),transformed(:,3),9,'red');
               set(gca,'xticklabel',[]);
               set(gca,'yticklabel',[]);
               set(gca,'zticklabel',[]);
               axis equal; axis vis3d;
               grid off;
               view(60,40);
               err=error_cal(target,transformed);
               title(['SoftAssign error is:',num2str(err)]);
               error_SoftAssign(j,i)=err;
 
               [transformed]=PCA_alignment(source,target,0);
               subplot(1,5,5); 
               scatter3(target(:,1),target(:,2),target(:,3),10,'filled','black'); hold on;
               scatter3(transformed(:,1),transformed(:,2),transformed(:,3),9,'red');
               set(gca,'xticklabel',[]);
               set(gca,'yticklabel',[]);
               set(gca,'zticklabel',[]);
               axis equal; axis vis3d;
               grid off;
               view(60,40);
               err=error_cal(target,transformed);
               title(['Our method error is:',num2str(err)]);
               error_PCA(j,i)=err;

 
               % -----------------------------save all images of the alignment----------------------------
           if save_file==1   
               tmp=getframe(gcf);
               image=tmp.cdata;
               imwrite(image, [current_path,'/motion_experiment/',filename(j,1).name,'_alignment_',num2str(i),'_motion,png'],'png');
           end;
           
       end;   
end;  
end; 
save('ICP_motion_error.txt','error_ICP','-ascii');
save('GM_motion_error.txt','error_GM','-ascii');
save('SoftAssign_motion_error.txt','error_SoftAssign','-ascii');
save('PCA_motion_error.txt','error_PCA','-ascii');
end
